/*
Copyright 2020 The Knative Authors

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package v1

import (
	"context"

	"k8s.io/apimachinery/pkg/api/equality"
	"knative.dev/pkg/apis"
	duckv1 "knative.dev/pkg/apis/duck/v1"
)

func isChannelEmpty(f duckv1.KReference) bool {
	return equality.Semantic.DeepEqual(f, duckv1.KReference{})
}

// Valid if it is a valid object reference.
func isValidChannel(ctx context.Context, f duckv1.KReference) *apis.FieldError {
	errs := f.Validate(ctx)

	// Namespace field is disallowed
	if f.Namespace != "" {
		fe := apis.ErrDisallowedFields("namespace")
		fe.Details = "only name, apiVersion and kind are supported fields"
		errs = errs.Also(fe)
	}

	return errs
}
